/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2007,2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
dojo.provide("zero.assemble.form.resource.WebUIActivityRestStore");

dojo.require("zero.resource.RestStore");

dojo.declare("zero.assemble.form.resource.WebUIActivityRestStore", zero.resource.RestStore, {
	
	// Summary:
	// 		This data store is for webUI activity only. where it behaves like this:
	//  	1. when performing fetch on data store, an array that contains only one
	//		   item is returned where the item is activity input data.
	// 		2. when performing save/newItem on data store, it POSTs data to activity URL.			
	
	_REST_URL_SURFFIX : "?data",
	
	resourceUri : "",
	
	constructor : function() {
		this._processUri();
	},
	
	_processUri : function() {
		// Summary:
		// 		ensure the resource url is legal.
		
		if(!this.resourceUri) {
			// comply REST HTTP API of webUI activity
			this.resourceUri = location.pathname + this._REST_URL_SURFFIX;
		
		} 
		// if not presenting "?data"
		else if (this.resourceUri.indexOf(this._REST_URL_SURFFIX, this.resourceUri.length - 5) < 0) {
			this.resourceUri += this._REST_URL_SURFFIX;
		}
	},
	
	_processItems : function(activityData) {
		// Summary:
		// 		override method to return array instead of single item only to 
		// 		comply REST URI convention.
		//		see zero.resource.RestStore._processItems()
		//	activityData:
		//		activity input data.
		
		var items = [];
		if(activityData === null) {
			console.warn("fetched nothing from flow");
		} else {
			// not necessary
			// this._arrayOfItems = [];
			var item = activityData;
			item[this._storeRef] = this;
			items.push(item);
		}
		return items;
	},
	
	newItem: function(obj, parentInfo) { 
		// Summary:
		//		add 200 as valid status code.
		//		see zero.resource.RestStore.newItem()
		
		var deferred = this._performPost(obj);
        
		obj[this._storeRef] = this;
        this.onNew(obj,parentInfo);
		return obj;
	},
	
	save: function(/* object */ keywordArgs) {
		// Summary:
		//		save changed items.
		//		see zero.resource.RestStore.save()
		
		var scope = keywordArgs.scope || dojo.global;
		 var item = null, obj = null;
		 
		 console.debug(this._changedItems);
		 
		 for(var id in this._changedItems){
		 	item = this._changedItems[id];
		 	obj = {};
		 	for(var key in item){
		 		if(key != this._storeRef){
		 			obj[key] = item[key];
		 		}
		 	}
		 	
		 	var deferred = this._performPost(obj);
		 }
		 if (keywordArgs.onComplete) {
			 keywordArgs.onComplete.call(scope);
            this._changedItems = {};
		 } else if (keywordArgs.onError) {
            keywordArgs.onError.call(scope);
        }
	},
	
	fetchItemByIdentity: function(keywordArgs) {
		var identity = keywordArgs.identity;
		console.debug(identity);
		// ignore identity based on webUI activity HTTP API
		var args = {
  			url: this.resourceUri,
			handleAs: "json-comment-optional",
			preventCache: true
		};
		
		var self = this;
		var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
		var deferred = dojo.xhrGet(args);
		deferred.addCallback(function(data) {
			var item = null;
			if (data !== null) {
				item = data;
				item[self._storeRef] = self;
			}
			if(keywordArgs.onItem){
				keywordArgs.onItem.call(scope, item);
			}
		});
		deferred.addErrback(function(error){
			if(keywordArgs.onError){
				keywordArgs.onError.call(scope, error);
			}
		});
	},
	
	deleteItem: function(item) {
		throw new Error("WebUIActivityRestStore doesn't support deletion.");
	},
	
	_performPost : function(/* object */ obj) {
		// Summary:
		// 		perform post request and change passed in data if necessary.
		// 		add 200 as valid status code.
		//	obj:
		//		posted data object.
		
		var self = this;
        var err = null;
		var args = {
				url: this.resourceUri,
				handleAs: "json-comment-optional",
				postData: dojo.toJson(obj),
				preventCache: true,
				sync: true,
			    contentType: "application/json",
	            handle: function(resp, ioArgs) {
	                //IE returns 1223 on a 204 response.  Argh.
	                if(ioArgs.xhr.status === 1223 || ioArgs.xhr.status === 204 || ioArgs.xhr.status === 200){
	                    // do nothing.
	                }else {
	                    err = new Error(ioArgs.xhr.status + ": " + ioArgs.xhr.statusText);
	                    err.responseText = ioArgs.xhr.responseText;
	                }
	            }
			};
		if(err){
            throw err;
        }
		return dojo.rawXhrPost(args);
	}
	
});